@@ -443,6 +443,23 @@ class Panoply:
 
         return self.xapi.xml_result()
 
+    def ad_hoc(self, qs: str) -> str:
+        """
+        Runs an ad_hoc command against this device.
+
+        example: type=op&action=complete&xpath=/operations/show/config/saved
+
+        :param qs: qs
+        :return: the unparsed xml document from the device API
+        """
+        try:
+            self.xapi.ad_hoc(qs=qs, modify_qs=True)
+
+        except PanXapiError as e:
+            raise PanoplyException(f"Could not execute query_set: {qs}: {e}")
+
+        return self.xapi.xml_document
+
     def fetch_license(self, auth_code: str) -> bool:
         """
         Fetch and install licenses for PAN-OS NGFW
@@ -657,6 +674,105 @@ class Panoply:
 
         return facts
 
+    def get_extended_facts(self) -> dict:
+        """
+        Get extended facts about the device to which we are connected.
+
+        Return interfaces, security_rules, and zones
+
+        :return: dict with extended facts
+        """
+
+        ef = {
+            "interfaces": self.get_interfaces(),
+            "security_rules": self.get_security_rules(),
+            "zones": self.get_zones()
+        }
+
+        return ef
+
+    def get_interfaces(self) -> dict:
+        """
+        Return a dict of all interfaces configured on this device. The dict has the following structure:
+
+        {
+          "ifnet": {
+            "entry": [
+              {
+                "name": "ethernet1/1",
+                "zone": "internet",
+                "fwd": "vr:default",
+                "vsys": "1",
+                "dyn-addr": null,
+                "addr6": null,
+                "tag": "0",
+                "ip": "10.48.58.161/23",
+                "id": "16",
+                "addr": null
+              }
+            ]
+          },
+          "hw": {
+            "entry": [
+              {
+                "name": "ethernet1/1",
+                "duplex": "full",
+                "type": "0",
+                "state": "up",
+                "st": "10000/full/up",
+                "mac": "fa:16:3e:65:7d:05",
+                "mode": "(autoneg)",
+                "speed": "10000",
+                "id": "16"
+              }
+            ]
+          }
+        }
+
+        :return: dict with two keys 'ifnet' and 'hw'.
+        """
+
+        interfaces_xml = self.execute_op('<show><interface>all</interface></show>', parse_result=False)
+
+        if self.xapi.status != 'success':
+            raise PanoplyException('Could not get interfaces!')
+
+        interfaces_result = xmltodict.parse(interfaces_xml)
+        return interfaces_result.get('response', {}).get('result', {})
+
+    def get_zones(self) -> list:
+        """
+        Return the list of configured zones on this device.
+
+        * returns a blank list for Panorama devices!
+
+        :return: list of zone names
+        """
+        if self.facts["model"] != 'Panorama':
+            return self.__get_completion(xpath='/operations/show/zone-protection/zone', completion_type='op')
+
+        else:
+            logger.info('not returning zones for Panorama')
+            return []
+
+    def get_security_rules(self) -> list:
+        """
+        Return the list of configured security rules on this device.
+
+        * returns a blank list for Panorama devices!
+
+        :return: list of security rules
+        """
+        if self.facts["model"] != 'Panorama':
+            return self.__get_completion(xpath='/config/devices/entry/vsys/entry/rulebase/security/rules',
+                                         completion_type='config')
+        else:
+            logger.info('not returning security rules for Panorama')
+            return []
+
+    def debug_completion(self, xpath, ct) -> list:
+        return self.__get_completion(xpath=xpath, completion_type=ct)
+
     def load_baseline(self) -> bool:
         """
         Load baseline config that contains ONLY connecting username / password
@@ -1116,10 +1232,8 @@ class Panoply:
             return self.get_configuration_version(config_source)
 
         else:
-            if self.facts.get("panorama-server", None) is not None:
-                cmd = "show config merged"
-            else:
-                cmd = "show config running"
+            # fixme - check for panorama and possibly do something differently
+            cmd = "show config running"
 
         try:
 
@@ -1474,6 +1588,47 @@ class Panoply:
 
         return self.__order_set_commands(diffs)
 
+    def __get_completion(self, xpath: str, completion_type=None) -> list:
+        """
+        Utility method to query the device for a completion value.
+
+        Example: To get list of zone names, use the following xpath: /operations/show/zone-protection/zone
+        This is equivalent to typing 'show zone-protection zone' and hitting tab on the CLI
+
+        :param xpath: xpath to the cli command from which to request a completion
+        :param completion_type: completion type: op, config, set. Default is 'op'
+        :return: list of completion values
+        """
+
+        completions = list()
+        try:
+            if not self.connected:
+                return completions
+
+            if completion_type is None:
+                completion_type = 'op'
+
+            if completion_type == 'config':
+                self.xapi.ad_hoc(qs=f"type={completion_type}&action=show&action=complete&xpath={xpath}", modify_qs=True)
+            else:
+                self.xapi.ad_hoc(qs=f"type={completion_type}&action=complete&xpath={xpath}", modify_qs=True)
+
+            if self.xapi.status != "success":
+                raise PanoplyException(f"Could not list completions from the device for xpath: {xpath}")
+
+            response_root = self.xapi.element_root
+            config_items = response_root.findall(".//completion")
+            for config in config_items:
+                if "value" in config.attrib:
+                    completions.append(config.attrib["value"])
+
+            return completions
+
+        except PanXapiError as pe:
+            logger.error("Could not list completions from device")
+            logger.error(pe)
+            raise PanoplyException(f"Could not list completions from the device for xpath: {xpath}")
+
     def __check_element(self, el: etree.Element, xpath: str, pc: etree.Element, not_founds: list) -> list:
         """
         recursive function to determine if the 'el' Element found at 'xpath' can also be found at the same
@@ -1690,7 +1845,7 @@ class Panoply:
             if leaf_xpath.startswith(xpath):
                 # note we do not remove found snippets from the source snippets list, which may result
                 # in duplicates. The calling code will need to ensure it does not append the results of this method
-                # which out checking for dups first
+                # with out checking for dups first
                 filtered_snippets.append(s)
 
         return filtered_snippets
