@@ -13,7 +13,7 @@
 # OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 # Authors: Nathan Embery
-
+import ipaddress
 import json
 import logging
 import re
@@ -21,12 +21,15 @@ import xml.etree.ElementTree as elementTree
 from abc import ABC
 from abc import abstractmethod
 from base64 import urlsafe_b64encode
+from collections import OrderedDict
 from copy import deepcopy
 from typing import Any
 from typing import Tuple
+from uuid import uuid4
 from xml.etree.ElementTree import ParseError
 
 import jmespath
+from jmespath import functions as jmespath_functions
 import xmltodict
 from jinja2 import BaseLoader
 from jinja2 import Environment
@@ -39,6 +42,7 @@ from jsonpath_ng import parse
 from lxml import etree
 from passlib.hash import md5_crypt
 
+from skilletlib.exceptions import NodeNotFoundException
 from skilletlib.exceptions import SkilletLoaderException
 from skilletlib.exceptions import SkilletValidationException
 
@@ -518,8 +522,8 @@ class Snippet(ABC):
             return meta.find_undeclared_variables(parsed_template_str)
 
         except TemplateError as te:
-            logger.error('Could not parse template string in get_variables_from_template')
-            raise SkilletValidationException(f'Error Parsing template {te}')
+            logger.error(f'Template Error in snippet: {self.name}')
+            raise SkilletValidationException(f'Error Parsing template in snippet: {self.name}: {te}')
 
     def get_output_variables(self) -> list:
         """
@@ -548,11 +552,13 @@ class Snippet(ABC):
                     test_str = "{{ " + str(self.metadata[i]) + " }}"
                 else:
                     test_str = self.metadata[i]
-                found_vars = self.get_variables_from_template(test_str)
 
-                for f in found_vars:
-                    if f not in variables:
-                        variables.append(f)
+                if test_str:
+                    found_vars = self.get_variables_from_template(test_str)
+
+                    for f in found_vars:
+                        if f not in variables:
+                            variables.append(f)
 
         return variables
 
@@ -632,7 +638,7 @@ class Snippet(ABC):
 
     # define functions for custom jinja filters
     @staticmethod
-    def __md5_hash(txt: str) -> str:
+    def _md5_hash(txt: str) -> str:
         """
         Returns the MD5 Hashed secret for use as a password hash in the PAN-OS configuration
 
@@ -644,20 +650,38 @@ class Snippet(ABC):
         return md5_crypt.hash(txt)
 
     @staticmethod
-    def __json_query(obj: dict, query: str) -> Any:
+    def _json_query(obj: dict, query: str) -> Any:
         """
         JMESPath query, jmespath.org for examples
 
         :param query: JMESPath query string
         :param obj: object to be queried
         """
+
+        class CustomFunctions(jmespath_functions.Functions):
+            """Custom jmespath functions"""
+
+            @jmespath_functions.signature({'types': ['string']})
+            def _func_remove_proto(self, s):
+                """Remove protocols from URL's, such as https://"""
+                if re.search("^[a-zA-Z]{0,5}://", s):
+                    return s[s.index('//') + 2:]
+                return s
+
+            @jmespath_functions.signature({'types': ['string']}, {'types': ['string']})
+            def _func_regex_matches(self, s, r):
+                """Remove protocols from URL's, such as https://"""
+                return bool(re.search(r, s))
+
+        options = jmespath.Options(custom_functions=CustomFunctions())
+
         if not isinstance(query, str):
             raise SkilletLoaderException('json_query requires an argument of type str')
-        path = jmespath.search(query, obj)
+        path = jmespath.search(query, obj, options=options)
         return path
 
     @staticmethod
-    def __slugify(txt: str) -> str:
+    def _slugify(txt: str) -> str:
 
         txt = re.sub(r'\s+', '_', txt)
         txt = re.sub(r'[^\w+\-]', '', txt)
@@ -668,6 +692,299 @@ class Snippet(ABC):
 
         return txt
 
+    def _node_attribute_present(self, obj: dict, config_path: str, attribute_name: str, attribute_value: str) -> bool:
+        """
+        Ensure a node with the named attribute and value does exist
+
+        :param obj: obj to check
+        :param config_path: path or child attribute(s) to check
+        :param attribute_name: name of the attribute
+        :param attribute_value: name of the value
+        :return: bool
+        """
+        if not attribute_name.startswith('@'):
+            attribute_name = f'@{attribute_name}'
+
+        try:
+            parent_obj = self._get_value_from_path(obj, config_path)
+        except SkilletLoaderException:
+            return False
+        except NodeNotFoundException:
+            return False
+
+        if type(parent_obj) is OrderedDict or type(parent_obj) is dict:
+            if attribute_name in parent_obj:
+                if attribute_value == parent_obj[attribute_name]:
+                    return True
+        elif type(parent_obj) is list:
+            for p in parent_obj:
+                if attribute_name in p:
+                    if attribute_value == p[attribute_name]:
+                        return True
+
+        return False
+
+    def _node_attribute_absent(self, obj: dict, config_path: str, attribute_name: str, attribute_value: str) -> bool:
+        """
+        Ensure a node with the named attribute and value does not exist
+
+        :param obj: obj to check
+        :param config_path: path or child attribute(s) to check
+        :param attribute_name: name of the attribute
+        :param attribute_value: name of the value
+        :return: bool
+        """
+
+        if self._node_attribute_present(obj, config_path, attribute_name, attribute_value):
+            return False
+
+        return True
+
+    def _node_present(self, obj: dict, config_path: str) -> bool:
+        try:
+            self._get_value_from_path(obj, config_path)
+            return True
+        except NodeNotFoundException:
+            return False
+        except SkilletLoaderException:
+            return False
+
+    def _node_value(self, obj: dict, config_path: str) -> Any:
+        try:
+            return self._get_value_from_path(obj, config_path)
+        except NodeNotFoundException:
+            return None
+        except SkilletLoaderException:
+            return None
+
+    def _node_value_contains(self, obj: dict, config_path: str, val_to_test: str) -> bool:
+
+        try:
+            val = self._get_value_from_path(obj, config_path)
+            if type(val) is str:
+                return val == val_to_test
+            elif type(val) is list or type(val) is dict or type(val) is OrderedDict:
+                return val_to_test in val
+
+            return False
+        except NodeNotFoundException:
+            return False
+        except SkilletLoaderException:
+            return False
+
+    def _get_value_from_path(self, obj: dict, config_path: str) -> Any:
+
+        if type(obj) is not dict and type(obj) is not OrderedDict:
+            logger.debug("Supplied object is not an Object")
+            logger.debug('Ensure you are passing an object here and not a string as from capture_pattern')
+            raise SkilletLoaderException('Incorrect object format for get_value_from_path')
+
+        if '.' in config_path or '/' in config_path:
+            if '.' in config_path:
+                separator = '.'
+            else:
+                separator = '/'
+            path_elements = config_path.split(separator)
+            first_path_element = path_elements[0]
+            p0 = self._check_inner_object(obj, first_path_element)
+            for p in path_elements:
+                if self._has_child_node(p0, p):
+                    new_p0 = p0[p]
+                    p0 = new_p0
+                else:
+                    raise NodeNotFoundException(f'{config_path} not found!')
+
+            return p0
+
+        p0 = self._check_inner_object(obj, config_path)
+
+        if self._has_child_node(p0, config_path):
+            return p0[config_path]
+        else:
+            raise NodeNotFoundException(f'{config_path} not found!')
+
+    def _node_absent(self, obj, child_key) -> bool:
+
+        out = self._node_present(obj, child_key)
+        if out:
+            return False
+
+        return True
+
+    @staticmethod
+    def _append_uuid(string_input: str) -> str:
+        """
+        Simple filter to append a generated UUID to the end of the given string
+
+        :param string_input: string to which to append the UUID
+        :return: string with a dash and a string UUID
+        """
+        new_uuid = str(uuid4())
+        return f'{string_input}-{new_uuid}'
+
+    @staticmethod
+    def _check_inner_object(obj: dict, child: str) -> dict:
+        """
+        Check inner object for named child dict key
+        We often get a dict object which contains a single key. The value of this key is itself a dict
+        and we need to know if the child key name exists on either the outer dict or the inner dict
+        return the inner dict if the child key name is found there, or punt and return the outer dict
+        otherwise
+
+        :param obj: dict to check for child key name
+        :param child: name of a key we want to find
+        :return: inner dict if it contains the child key, outer dict otherwise
+        """
+        if child in obj:
+            return obj
+
+        if len(obj.keys()) == 1:
+            inner_obj = obj[list(obj)[0]]
+            if child in inner_obj:
+                return inner_obj
+
+        return obj
+
+    def _verify_in_list(self, list1: list, list2: (list, dict), list2_path='.') -> bool:
+        """
+        Iterate all items from the list. Verify they are present in at least one item
+        from the second list. In the case where list2 is a list of objects / dictionaries
+        the list2_path can be used to determine if the item from list1 is present.
+
+        Consider a list of objects, each with a list of members. Verify all items from list1
+        exist in the members list of at least one object.
+
+        :param list1: list of items to verify exist in list2
+        :param list2: list of items or objects to check
+        :param list2_path: when list2 contains objects, the path to check for each object for the presence list1 item
+        :return: boolean true if all items from list1 are present in list2
+        """
+        for item in list1:
+
+            found = False
+
+            for second_item in list2:
+                if list2_path == '.':
+                    item2 = second_item
+                else:
+                    item2 = self._get_value_from_path(second_item, list2_path)
+
+                if isinstance(item2, list):
+                    if item in item2:
+                        found = True
+                        break
+                elif isinstance(item2, str):
+                    if item == item2:
+                        found = True
+                        break
+
+            if not found:
+                return False
+
+        return True
+
+    @staticmethod
+    def _verify_item_in_list(obj: Any, list2: (list, dict)) -> bool:
+        """
+        Similar to __verify_items_in_list, except checks if a string, or list member
+        is present in list2
+
+        :param obj: obj to test if in list2
+        :param list2: list to test if obj is a member of
+        """
+        if isinstance(obj, list):
+            for o in obj:
+                if o in list2:
+                    return True
+            return False
+        return obj in list2
+
+    @staticmethod
+    def _permitted_address(obj: dict, permitted: list) -> bool:
+        """
+        Check if a NGFW formatted address object is in a list of permissible ranges
+
+        :param obj: NGFW formatted address entry as a dict
+        :param permitted: List of permissible address objects in CIDR format
+        """
+        test_network = ipaddress.ip_network(obj['entry']['ip-netmask'])
+        for network in permitted:
+            if test_network.subnet_of(ipaddress.ip_network(network)):
+                return True
+
+        return False
+
+    @staticmethod
+    def _difference(list1: list, list2: list) -> list:
+        """
+        Returns a list of items from list1 that do not exist in list2
+
+        :param list1: list of items expected to be in list2
+        :param list2: list of items list1 will be evaluated against
+        """
+        if not isinstance(list1, list) or not isinstance(list2, list):
+            raise SkilletLoaderException('difference filter takes only type list for both arguments.')
+
+        return list(set([x for x in list1 if x not in list2]))
+
+    @staticmethod
+    def _listify(obj: Any) -> list:
+        """
+        Attempt to convert a string input into a list
+
+        :param obj: raw input text
+        """
+        if isinstance(obj, list):
+            return obj
+
+        if isinstance(obj, str):
+            if '\n' in obj:
+                return [x.strip() for x in obj.split('\n') if x]
+            elif ',' in obj:
+                return [x.strip() for x in obj.split(',') if x]
+            return [obj.strip()]
+
+        raise SkilletLoaderException('listify filter requires input of type str.')
+
+    @staticmethod
+    def _has_child_node(obj, node_name) -> bool:
+        if obj is None:
+            return False
+
+        if type(obj) is not dict and type(obj) is not OrderedDict:
+            return False
+
+        if node_name in obj:
+            return True
+
+        return False
+
+    @staticmethod
+    def _minimal_indent(input_str: str, minimal_indent=4) -> str:
+        """
+        Ensures we have a minimal amount of tab ident for each line in the input_string
+
+        :param input_str: string to check for indents
+        :param minimal_indent: number of spaces required at the beginning of each file
+        :return: indented string
+        """
+
+        indented_str = ""
+        prepend = " " * int(minimal_indent)
+
+        for index, line in enumerate(input_str.split('\n')):
+            if index == 0:
+                indented_str += line + "\n"
+                continue
+
+            expanded_line = line.expandtabs(2)
+            if not expanded_line.startswith(prepend):
+                indented_str += f"{prepend}{expanded_line}\n"
+            else:
+                indented_str += expanded_line + "\n"
+
+        return indented_str
+
     def __init_env(self) -> None:
         """
         init the jinja2 environment and add any required filters
@@ -675,10 +992,32 @@ class Snippet(ABC):
         :return: Jinja2 environment object
         """
         self._env = Environment(loader=BaseLoader, extensions=[AnsibleCoreFiltersExtension])
-        self._env.filters["md5_hash"] = self.__md5_hash
-        self._env.filters["slugify"] = self.__slugify
-        self._env.filters["s"] = self.__slugify
-        self._env.filters['json_query'] = self.__json_query
+        self._env.filters["md5_hash"] = self._md5_hash
+        self._env.filters["slugify"] = self._slugify
+        self._env.filters["s"] = self._slugify
+        self._env.filters['json_query'] = self._json_query
+        self._env.filters['node_present'] = self._node_present
+        self._env.filters['node_absent'] = self._node_absent
+        self._env.filters['node_value'] = self._node_value
+        self._env.filters['node_value_contains'] = self._node_value_contains
+        self._env.filters['node_attribute_present'] = self._node_attribute_present
+        self._env.filters['node_attribute_absent'] = self._node_attribute_absent
+        self._env.filters['append_uuid'] = self._append_uuid
+        self._env.filters['minimal_indent'] = self._minimal_indent
+
+        # for zube ticket #21
+        self._env.filters['tag_present'] = self._node_present
+        self._env.filters['tag_absent'] = self._node_absent
+        self._env.filters['element_value'] = self._node_value
+        self._env.filters['element_value_contains'] = self._node_value_contains
+        self._env.filters['attribute_present'] = self._node_attribute_present
+        self._env.filters['attribute_absent'] = self._node_attribute_absent
+        self._env.filters['items_present'] = self._verify_in_list
+        self._env.filters['item_present'] = self._verify_item_in_list
+        self._env.filters['permitted_address'] = self._permitted_address
+        self._env.filters['listify'] = self._listify
+        self._env.filters['difference'] = self._difference
+
         self.add_filters()
 
     def add_filters(self) -> None:
